@@ -30,6 +30,7 @@
 #include <libgossip/gossip-vcard.h>
 #include <libgossip/gossip-chatroom-provider.h>
 #include <libgossip/gossip-ft-provider.h>
+#include <libgossip/gossip-session.h>
 
 #include "gossip-jid.h"
 #include "gossip-jabber-chatrooms.h"
@@ -67,7 +68,10 @@
  * the user has stopped composing.
  */
 #define COMPOSING_TIMEOUT          45
-  
+
+/* How many rand char should be happend to the resource */
+#define N_RAND_CHAR                6
+
 struct _GossipJabberPriv {
 	LmConnection          *connection;
 	
@@ -321,6 +325,11 @@ static JabberData *     jabber_data_new                     (GossipJabber
 							     gpointer                 user_data);
 static void             jabber_data_free                    (JabberData              *data);
 
+static const gchar *server_conversions[] = {
+	"gmail.com", "talk.google.com",
+	"googlemail.com", "talk.google.com",
+	NULL
+};
 
 G_DEFINE_TYPE_WITH_CODE (GossipJabber, gossip_jabber, GOSSIP_TYPE_PROTOCOL,
 			 G_IMPLEMENT_INTERFACE (GOSSIP_TYPE_CHATROOM_PROVIDER,
@@ -733,7 +742,12 @@ jabber_connection_open_cb (LmConnection *connection,
 	gchar            *id;
 	const gchar      *resource;
 	gchar            *password = NULL;
-	
+#ifdef USE_RAND_RESOURCE
+	gchar            *resource_rand;
+	gchar             rand_str[N_RAND_CHAR + 1];
+	gint              i;
+#endif
+
 	priv = GET_PRIV (jabber);
 
 	if (priv->disconnect_request) {
@@ -793,12 +807,34 @@ jabber_connection_open_cb (LmConnection *connection,
 		return;
 	}
 
+#ifdef USE_RAND_RESOURCE
+	/* appens a random string to the resource to avoid conflicts */
+	for (i = 0; i < N_RAND_CHAR; i++) {
+		if (g_random_boolean()) {
+			rand_str[i] = g_random_int_range('A', 'Z' + 1);
+		} else {
+			rand_str[i] = g_random_int_range('0', '9' + 1);
+		}
+	}
+	rand_str[N_RAND_CHAR] = '\0';
+	resource_rand = g_strdup_printf ("%s.%s", resource, rand_str);
+
+	lm_connection_authenticate (priv->connection,
+				    id,
+				    password, 
+				    resource_rand,
+				    (LmResultFunction) jabber_connection_auth_cb,
+				    jabber, NULL, NULL);
+
+	g_free (resource_rand);
+#else
 	lm_connection_authenticate (priv->connection,
 				    id,
 				    password, 
 				    resource,
 				    (LmResultFunction) jabber_connection_auth_cb,
 				    jabber, NULL, NULL);
+#endif
 
 	g_free (id);
 	g_free (password);
@@ -880,7 +916,8 @@ jabber_disconnect_cb (LmConnection       *connection,
 		      LmDisconnectReason  reason,
 		      GossipJabber       *jabber)
 {
-	GossipJabberPriv *priv;
+	GossipJabberPriv       *priv;
+	GossipDisconnectReason  gossip_reason;
 
 	priv = GET_PRIV (jabber);
 
@@ -895,8 +932,22 @@ jabber_disconnect_cb (LmConnection       *connection,
 					     (GHRFunc) jabber_logout_contact_foreach,
 					     jabber);
 	}
-
-	g_signal_emit_by_name (jabber, "logged-out", priv->account);
+	
+	switch (reason) {
+	case LM_DISCONNECT_REASON_OK:
+		gossip_reason = GOSSIP_DISCONNECT_ASKED;
+		break;
+	case LM_DISCONNECT_REASON_PING_TIME_OUT:
+	case LM_DISCONNECT_REASON_HUP:
+	case LM_DISCONNECT_REASON_ERROR:
+	case LM_DISCONNECT_REASON_UNKNOWN:
+		gossip_reason = GOSSIP_DISCONNECT_ERROR;
+		break;
+	default:
+		g_assert_not_reached();
+	}
+	
+	g_signal_emit_by_name (jabber, "logged-out", priv->account, gossip_reason);
 }
 
 static LmSSLResponse 
@@ -1309,6 +1360,7 @@ jabber_get_default_server (GossipProtocol *protocol,
 	GossipJID        *jid;
 	const gchar      *str;
 	gchar            *server;
+	gint              i = 0;
 
 	g_return_val_if_fail (username != NULL, NULL);
 
@@ -1317,6 +1369,16 @@ jabber_get_default_server (GossipProtocol *protocol,
 	
 	jid = gossip_jid_new (username);
 	str = gossip_jid_get_part_host (jid);
+
+	while (server_conversions[i] != NULL) {
+		if (g_ascii_strncasecmp (str, server_conversions[i], -1) == 0) {
+			str = server_conversions[i + 1];
+			break;
+		}
+
+		i += 2;
+	}
+	
 	server = g_strdup (str);
 	gossip_jid_unref (jid);
 
@@ -1783,6 +1845,16 @@ jabber_contact_remove (GossipProtocol *protocol,
  	
 	lm_connection_send (priv->connection, m, NULL);
 	lm_message_unref (m);
+
+	/* Remove current subscription */
+	gossip_protocol_set_subscription (protocol, contact, FALSE);
+
+	/* Remove from internal hash table */
+	gossip_debug (DEBUG_DOMAIN, 
+		      "Contact:'%s' being removed with current ref count:%d", 
+		      gossip_contact_get_id (contact), G_OBJECT (contact)->ref_count);
+
+	g_hash_table_remove (priv->contacts, contact);
 }
 
 static void
@@ -2635,9 +2707,12 @@ jabber_request_version (GossipJabber *jabber,
 	r = lm_message_new_with_sub_type (from,
 					  LM_MESSAGE_TYPE_IQ,
 					  LM_MESSAGE_SUB_TYPE_RESULT);
-	lm_message_node_set_attributes (r->node,
-					"id", id,
-					NULL);
+	if (id) {
+		lm_message_node_set_attributes (r->node,
+						"id", id,
+						NULL);
+	}
+
 	info = gossip_version_info_get_own ();
 	node = lm_message_node_add_child (r->node, "query", NULL);
 	lm_message_node_set_attributes (node, 
@@ -2708,8 +2783,6 @@ jabber_request_roster (GossipJabber *jabber,
 							      &new_item, 
 							      FALSE);
 
-		g_object_set (contact, "type", GOSSIP_CONTACT_TYPE_CONTACTLIST, NULL);
-
 		/* Groups */
 		groups = NULL;
 		for (subnode = node->children; subnode; subnode = subnode->next) {
@@ -2751,6 +2824,26 @@ jabber_request_roster (GossipJabber *jabber,
 				type = GOSSIP_SUBSCRIPTION_NONE;
 			}
 
+			/* In the rare cases where we have this state,
+			 * NONE means that we are in the process of
+			 * setting up subscription so the contact is
+			 * still temporary, any other state and we
+			 * assume they must be a proper contact list
+			 * contact. 
+			 *
+			 * Also, later when we present the
+			 * subscription dialog to the user, we need to
+			 * know if user is temporary contact or an old
+			 * contact so we can silently accept
+			 * subscription requests for people already on
+			 * the roster with "to" or "from" conditions.
+			 */
+			if (type != GOSSIP_SUBSCRIPTION_NONE) {
+				g_object_set (contact, "type", 
+					      GOSSIP_CONTACT_TYPE_CONTACTLIST, 
+					      NULL);
+			}
+
 			gossip_contact_set_subscription (contact, type);
 		} 
 		
@@ -3208,7 +3301,9 @@ gossip_jabber_get_contact_from_jid (GossipJabber *jabber,
 				       gossip_jid_get_without_resource (jid));
 
 	if (!contact) {
-		gossip_debug (DEBUG_DOMAIN, "New contact:'%s'", gossip_jid_get_full (jid));
+		gossip_debug (DEBUG_DOMAIN, 
+			      "New contact:'%s' (GOSSIP_CONTACT_TYPE_TEMPORARY)", 
+			      gossip_jid_get_full (jid));
 		contact = gossip_contact_new (GOSSIP_CONTACT_TYPE_TEMPORARY,
 					      priv->account);
 		gossip_contact_set_id (contact, 
@@ -3229,7 +3324,32 @@ gossip_jabber_get_contact_from_jid (GossipJabber *jabber,
 			jabber_contact_vcard (jabber, contact);
 		}
 	} else {
-		gossip_debug (DEBUG_DOMAIN, "Get contact:'%s'", gossip_jid_get_full (jid));
+		GossipContactType  type;
+		const gchar       *type_str;
+
+		type = gossip_contact_get_type (contact);
+
+		switch (type) {
+		case GOSSIP_CONTACT_TYPE_TEMPORARY:
+			type_str = "GOSSIP_CONTACT_TYPE_TEMPORARY";
+			break;
+		case GOSSIP_CONTACT_TYPE_CONTACTLIST:
+			type_str = "GOSSIP_CONTACT_TYPE_CONTACTLIST";
+			break;
+		case GOSSIP_CONTACT_TYPE_CHATROOM:
+			type_str = "GOSSIP_CONTACT_TYPE_CHATROOM";
+			break;
+		case GOSSIP_CONTACT_TYPE_USER:
+			type_str = "GOSSIP_CONTACT_TYPE_USER";
+			break;
+		default:
+			type_str = "unknown";
+			break;
+		}
+
+		gossip_debug (DEBUG_DOMAIN, 
+			      "Get contact:'%s', type:%d-->'%s'", 
+			      gossip_jid_get_full (jid), type, type_str);
 	}
 
 	gossip_jid_unref (jid);
